/*
      bootgen.c                                    Achim Freimann
                                                   Hannover
         Bootprogrammgenerator

  29.01.88  v1.0  - Bootprogramm in Bootsektor schreiben
  05.02.88  v1.1  - Biostreiber in erste Spur, ab Sektor 1 ff.
                    schreiben

  Mu mit HDBIOS und CBIND aus Heft 2/88 gelinkt werden !! 
  Lattice C V3.03.04
*/

#include <stdio.h>
#include <osbind.h>
#include <portab.h>

#define CODELENGTH (0x1b7 - 0x1e) /* max. Gre Bootprg. */
#define TRACKLENGTH 16            /* max. Anz. der Sekt.
                                     fr Biostreiber     */

main (argc,argv)
int argc;
char *argv[];
{
   int i,done;

   super_on(); 
   install();   /* Leseroutine im Treiber initialisieren */ 
   superoff(); 

   printf ("%cE\n\n Bootsektorgenerator\n",27);
   puts (" -------------------\n");

   done =-1;
   if (argc <3 || argc >5) error(6);
   for (i=1; i<argc; i+=2)
   {  if (*argv [i] =='t')
         done=gendriver(argv[i+1]);
      if (*argv [i] =='b')
         done=genboot(argv[i+1]);
   }
   if (done) error (7);
   getchar();           /* Auf Tastendruck warten */
}

int
gendriver (fname)
LONG fname;
{
   WORD fhandle;
   register LONG count;
   char buffer[512*TRACKLENGTH];

   for (count=0;count<512*TRACKLENGTH;count++)
      buffer[count] = 0;

   if ((fhandle=Fopen (fname,0)) <0) error (1,fname);
   count=Fseek (0,fhandle,2);   /* Dateilnge holen */
   Fseek (0,fhandle,0);         /* 'zurckspulen' */

   if (count > TRACKLENGTH *512)
   {  Fclose (fhandle);
      error (8,fname);  }

   if (Fread (fhandle,count,&buffer) <0) error (2,fname);
   Fclose (fhandle);
   wr_sect (1,&buffer,TRACKLENGTH);

   printf (" %s ist in Startspur geladen.\n",fname);
   return (0);
}

int
genboot (fname)
LONG fname;
{
   char buffer[512];
   WORD fhandle;
   LONG count;


   if ((fhandle = Fopen (fname,0)) < 0) error (1,fname);

   rd_sect (0,&buffer,1);

   count = testformat (fhandle,fname);

   if (Fread (fhandle,count,&buffer[0x1e]) < 0) error (2,fname);
   Fclose (fhandle);

   buffer[0] = 0x60;  buffer[1] = 0x1c; /* branch bootcode */
   Protobt (&buffer,-1,-1,1);
   wr_sect (0,&buffer,1);

   printf (" %s ist in Bootsektor kopiert.\n",fname);
   return (0);
}

LONG
testformat (fhandle,fname)
WORD fhandle;
LONG fname;
{
   struct fileheader
   {  WORD magic;
      LONG text;
      LONG data;
      LONG bss;
      LONG symb;
      BYTE filler[10];
   }fh;

   LONG count,relocinfo;
   relocinfo = 0;

   /* Testen, ob Data- oder Bss-Segment vorhanden
      und Lnge des Text-Segments berprfen   */

   if ( Fread (fhandle,0x1cL,&fh) < 0)      error (2,fname);
   if ( fh.data || fh.bss )                 error (3,fname);
   if ( (count=fh.text) >= CODELENGTH )     error (4,fname);

   /* Auf Loadertabelle positionieren, Test ob PC-Relativ
      und wieder auf Programmanfang positionieren      */

   Fseek (fh.text+fh.symb,fhandle,1);
   if ( Fread (fhandle,4L,&relocinfo) < 0)  error (2,fname);
   if ( relocinfo != 0 )                    error (5,fname);
   Fseek (0x1cL,fhandle,0);
   return (count);
}

error (errno,text)   /* Hier wird gemeckert ! */
int errno,text;
{
   switch (errno)
   { case 1:
       printf (" Wo ist denn mein '%s' ?\n",text);    break;
     case 2:
       printf (" %s: Lesefehler !\n,text");           break;
     case 3:
       printf (" %s: Data-/Bss-Segment nicht erlaubt !\n",text);
       break;
     case 4:
       printf (" %s: Textsegment zu gro !\n",text);  break;
     case 5:
       printf (" %s: Programm ist nicht PC-Relativ !\n",text);
       break;
     case 6:
       puts (" Gebrauch: bootgen [t name]/[b name]"); break;
     case 7:
       puts (" Mll in der Eingabezeile !!");         break;
     case 8:
       printf (" %s: Datei ist zu gro !\n",text);    break;
   }
   getchar();           /* Auf Tastendruck warten */
   exit (-1);
}

rd_sect (sector,buffer,count)
int sector,buffer,count;
{
   super_on ();
   h_read (0,buffer,count,sector,0);
   superoff ();
}

wr_sect (sector,buffer,count)
int sector,buffer,count;
{
   super_on ();
   h_write (0,buffer,count,sector,0);
   superoff ();
}


