*+
*       bootload.s
*
*         Ladeprogramm fr HD-Treiber mit OMTI-Kontroller.
*         Der Treiber mu in den Sektoren 1 bis (max.) 16 von 
*         Spur Null, Kopf null stehen!
*
*         Steht selbst im zweiten Teil des 1 KByte groen Disk-
*         buffers, da der Treiber noch den ersten Teil bentigt.
*
*         08.02.88   Achim Freimann  
*-
prg_len   equ     5              ; Lnge des Treibers (Sektoren)
bootdrv   equ     2              ; c: ist BootLw

gemdos    equ     1
dsetdrv   equ     $e
pexec     equ     $4b 
flock     equ     $43e           ; Floppy VBL sperren
_bootdev  equ     $446           ; Variable fr Bootlaufwerk
_dskbufp  equ     $4c6           ; Zeiger auf 1k Diskbuffer
_hz_200   equ     $4ba           ; 200Hz Zhler
the_env   equ     $4be           ; Std. Enviroment (Nullbytes)
second    equ     200            ; Faktor fr Sekunden
start     equ     -$20           ; Offset auf Sektoranfang

diskctl   equ     $ffff8604      ; Kontrollerzugriff
fifo      equ     $ffff8606      ; DMA-Steuerregister

          text

*+
*       Register Vorbesetzung, damit es krzer wird:
*
*         D5 = #$c8         Steuerwort fr DMA-Chip
*         D6 = #$ca                    "
*         D7 = #0           (Kommentar ?)
*         A3 = #0
*         A4 = fifo
*         A5 = diskctl
*-

          lea     diskctl,a5
          lea     (a5),a4
          addq.l  #2,a4          ; ist jetzt fifo
          moveq   #0,d7          ; D7 = #0
          sub.l   a3,a3          ; A3 = #0
          move    #$c8,d5        ; Steuerwort fr DMA-Chip
          move    #$ca,d6        ;            "

*    Basepage fr Treiber erzeugen ...
          lea     the_env(a3),a0 ; Default-Nullstring
          pea     (a0)           ; Enviroment
          pea     (a0)           ; Command
          pea     (a0)           ; Path
          move    #5,-(sp)
          move    #pexec,-(sp)   ; Basepage erzeugen
          trap    #gemdos
          lea     16(sp),sp      ; Rumkommando

          move.l  d0,a2
          add.l   #$100,d0       ; Beginn Textsegment
          move.l  d0,8(a2)       ; in Basepage eintragen

*    ... Treiber (oder sonstigen Schund) laden ...
          sub.l   #$1c,d0        ; Dateikopf abziehen
          move.l  d0,a1          ; Startadresse
          move.l  a1,-(sp)
          moveq   #1,d0          ; Startsektor: 1 
          moveq   #prg_len,d3    ; Anzahl Sekt.: prg_len
          bsr.s   hd_read        ; Sektoren lesen

          move.l  (sp)+,a1       ; Startadresse Programm
          move.l  2(a1),$c(a2)   ; TEXT-Lnge in Basepage

          lea     return(pc),a0
          move.l  (sp)+,(a0)     ; zwei Rcksprungadressen
          move.l  (sp)+,4(a0)    ;  merken ...

*    ... und ausfhren (exekutieren !)
          move.l  a2,-(sp)       ; Start Basepage
          pea     the_env(a3)    ; Dummy
          move    #4,-(sp)
          move    #pexec,-(sp)   ; Prg. starten
          trap    #gemdos
          lea     12(sp),sp

*    Bootlaufwerk fr Autoprogramme und Desktop setzen
*          move    #bootdrv,_bootdev(a3) ; siehe Text !!
          move    #bootdrv,-(sp)
          move    #dsetdrv,-(sp) ; Bootlaufwerk anmelden
          trap    #gemdos        ; (sonst Autoprg von Disk !)
          addq.l  #4,sp

          move.l  return+4(pc),-(sp)   ; ... und zurckholen
          move.l  return(pc),-(sp)
          rts

*+
*       Sektoren von HardDisk lesen (ohne DMA)
*
*       hd_read(buf,recno,count)
*
*         A1.l :  buf       (Zieladresse)
*         D0.l :  recno     (logischer Sektor)
*         D3.b :  count     (Anzahl Sektoren)
*-              
hd_read   st      flock(a3)
          lea     cmd_tab(pc),a0 ; Beginn Kommandotabelle
          move    d0,4(a0)       ; Startsektor
          move    d3,8(a0)       ; Anzahl der Sektoren
          bsr.s   send_cmd
          bne.s   rd_err

          subq    #1,d3          ; Sektorzhler
rd_sect   move    #511,d2        ; Bytezhler
rd_byte:  bsr.s   wait_req
          bne.s   rd_err
          move    (a5),d0
          move.b  d0,(a1)+
          dbra    d2,rd_byte     ; Sektor schon komplett ?
          bsr.s   wait_req
          bne.s   rd_err
          dbra    d3,rd_sect     ; nchsten Sektor lesen
          bsr     status         ; Status holen

rd_err    move    #$80,(a4)      ; DMA wieder auf Floppy
          sf      flock(a3)      ; Floppy-VBL zulassen
          rts

*+
*       Kommando senden
*-
send_cmd  bsr.s   select         ; Kontroller selektieren

          moveq   #5,d2          ; = 6 Kommandobytes
send      bsr.s   wait_req
          bne.s   send_err       ; timeout
          move    (a0)+,(a5)
          dbra    d2,send

          move.l  #2*second,d1   ; 2 Sek. timeout
          bsr.s   wait_l
send_err  rts

*+
*       Auf bertragungsanforderung warten
*
*         wait_req: max. 1/10 Sekunden warten
*         wait_l:   max. D1.l * 1/200 Sekunden warten
*
*         -> timeout : D0.l = -1
*            ok      : D0.l = 0
*-
wait_req  moveq.l #20,d1         ; 0.1 Sek.
wait_l    bsr.s   st_port        ; Statusport anwhlen
          add.l   _hz_200(a3),d1
_wait     move    (a5),d0
          and     #1,d0          ; REQ-Bit testen
          bne.s   wait_ok        ; gesetzt = NE
          cmp.l   _hz_200(a3),d1 ; Zeit abgelaufen ?
          bne.s   _wait
          bsr.s   d_port         ; Datenport anwhlen
          moveq   #-1,d0
          rts

wait_ok   bsr.s   d_port         ; Datenport anwhlen
          moveq   #0,d0
          rts

*+
*       Statusbyte lesen
*-
status    bsr.s   wait_req
          move    (a5),d0        ; Fehlerbyte lesen
          and     #$ff,d0
          tst     d0
          rts

*+
*       Kontroller selektieren
*-
select    bsr.s   sel_port
          move    d7,(a5)        ; Select
          bra.s   d_port

*+
*       Datenport whlen
*-
d_port    move    d6,(a4)
          move    d7,(a5)
          move    d5,(a4)
          rts

*+
*       Status-/Resetport whlen
*-
st_port   move    d6,(a4)
          move    #1,(a5)
          move    d5,(a4)
          rts

*+
*       Konfigurations-/Selectport whlen
*-
sel_port  move    d6,(a4)
          move    #2,(a5)
          move    d5,(a4)
          rts

*+
*       Einige Daten
*-
cmd_tab   dc.w    8,0,0,0,0,1    ; Befehlssequenz fr Lesen
return    dc.l    0,0            ; 2 Rcksprungadressen


