*+
*       cbind.s                                               Achim Freimann
*       Version  fr DMA-Port
*
*       Stackkorrektur fr Anbindung von Lattice C - Funktionsaufrufen
*       an Assemblerroutinen und Anbindung der Formatierroutinen
*
*       Der Lattice C bergibt nur int (=long) an Funktionen !!!
*
*       18.11.87   af   v1.0   - alle Funktionen ok.
*       16.01.88   af   v1.1   - der WD1002S - WX2 formatiert zu lahm ->
*                                bei Zeitberschreitung Rckfrage nehmen
*       18.01.88   af   v1.2   - h_read & h_write bergeben nun recno als
*                                32-Bit-Argument
*-

flock     equ     $43e                ; Variable zum sperren von FDC-Aktionen
diskctl   equ     $ffff8604           ; Kontrollerzugriff/Sektorzhler
fifo      equ     $ffff8606           ; DMA-Steuerregister
steprate  equ     1                   ; 10 mikrosec.
timeout   equ     180*200             ; 180 Sekunden Timeout
_format   equ     4                   ; Platte formatieren
_bufwr    equ     $f                  ; Kontrollerbuffer schreiben
_assign   equ     $11                 ; Ersatzspur zuweisen

gemdos    equ     1                   ; KEIN Kommentar !
cconin    equ     1
cconws    equ     9

          xdef    h_read
          xdef    h_write
          xdef    hd_form
          xdef    hd_bufwr
          xdef    hdassign
          xref    hd_read
          xref    hd_write
          xref    send_cmd
          xref    wait_req
          xref    wait_l
          xref    status

          text
          page
*+
*       Laufwerk formatieren
*
*         hd_form(interleave)
*           long interleave
*-
interl    set     10                  ; das ist untere Wort !
*
hd_form   link    a6,#0
          movem.l d1-d3/a4-a5,-(sp)
          lea     diskctl,a5
          lea     fifo,a4
          st      flock               ; Floppy VBL sperren

          move    #$40+steprate,-(sp) ; Kontroll-Byte
          move    interl(a6),-(sp)    ; Interleave-Faktor
          clr.l   -(sp)
          clr     -(sp)
          move    #_format,-(sp)
          jsr     send_cmd
          add.l   #12,sp

fmtdelay  move.l  #timeout,d1
          jsr     wait_l
          beq     form_ok
          bsr     err_req             ; Text ausg. & Antwort holen
          tst     d0
          bne     fmtdelay            ; noch mehr warten

form_ok   jsr     status              ; Statusbyte holen
          move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d3/a4-a5
          unlk    a6
          rts

          page
*+
*       Warnmeldung ausgeben und Besttigung holen
*-
err_req   pea     err_msg
          move    #cconws,-(sp)
          trap    #gemdos
          addq.l  #6,sp

_getkey   move    #cconin,-(sp)
          trap    #gemdos
          addq.l  #2,sp
          and     #$5f,d0             ; klein- >> GROSSschrift
          cmp.b   #'F',d0
          beq     _ok
          cmp.b   #'A',d0
          bne     _getkey             ; En attendant Godot
          bsr     crlf
          moveq.l #0,d0
          rts

_ok       bsr     crlf
          moveq.l #-1,d0
          rts

crlf      pea     _crlf
          move    #cconws,-(sp)
          trap    #gemdos
          addq.l  #6,sp
          rts

          data
err_msg   dc.b    '    Zeitberschreitung !   (F)ortsetzen oder (A)bbruch ? ',0
_crlf     dc.b    13,10,0
          even
          text

          page
*+
*       Kontrollerbuffer beschreiben
*
*       hd_bufwr(buf)
*         long buf 
*-
buf       set     8
*
hd_bufwr  link    a6,#0
          movem.l d1-d3/a4-a5,-(sp)
          lea     diskctl,a5
          lea     fifo,a4
          st      flock               ; Floppy VBL sperren
          clr     d1

          clr     -(sp)               ; Kontroll-Byte
          move    #1,-(sp)            ; Blockzhler
          clr.l   -(sp)
          clr     -(sp)
          move    #_bufwr,-(sp)
          jsr     send_cmd
          bne     bufwr_er            ; -> timeout
          add.l   #12,sp

          clr     d0
          move.l  buf(a6),a0
          move    #511,d3             ; Bytezhler
bwr_byte  jsr     wait_req
          bne     bufwr_er            ; -> timeout
          move.b  (a0)+,d0
          move    d0,(a5)
          dbf     d3,bwr_byte
          bsr     status              ; Statusbyte lesen

bufwr_er  move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d3/a4-a5
          unlk    a6
          rts

          page
*+
*       Ersatzspur zuweisen
*
*       hdassign(ohead,ocyl,nhead,ncyl,interl)
*         long ohead,ocyl,nhead,ncyl,interl
*-
ohead     set     10
ocyl      set     14
nhead     set     18
ncyl      set     22
interl    set     26
*
hdassign  link    a6,#0
          movem.l d1-d3/a4-a5,-(sp)
          lea     diskctl,a5
          lea     fifo,a4
          st      flock               ; Floppy VBL sperren
          clr     d1

          move    #$40+steprate,-(sp) ; Kontroll-Byte
          move    interl(a6),-(sp)    ; Interleave-Faktor
          move    ocyl(a6),d0
          clr     -(sp)
          move.b  d0,1(sp)            ; C7 - C0
          clr.b   d0
          lsr     #2,d0
          move    d0,-(sp)            ; C9 - C8
          move    ohead(a6),-(sp)     ; Kopf
          move    #_assign,-(sp)
          jsr     send_cmd
          bne     assi_err            ; -> timeout
          add.l   #12,sp

          move    ncyl(a6),d0
          clr.l   -(sp)
          move.b  d0,1(sp)            ; C7 - C0
          clr.b   d0
          lsr     #2,d0
          move    d0,-(sp)            ; C9 - C8
          move    nhead(a6),-(sp)     ; Kopf
          move    #3,d3               ; Bytezhler
ass_byte  jsr     wait_req
          bne     assi_err            ; -> timeout
          move    (sp)+,(a5)
          dbf     d3,ass_byte

          move.l  #30*200,d1          ; 30 Sek. timeout
          bsr     wait_l
          bne     assi_err
          bsr     status              ; Statusbyte lesen

assi_err  move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d3/a4-a5
          unlk    a6
          rts

*+
*       Sektoren lesen
*-
h_read    move.l  8(sp),6(sp)         ; buf
          move    14(sp),10(sp)       ; count.l = count.w
          move.l  16(sp),12(sp)       ; recno.l = recno.l
          jmp     hd_read

*+
*       Sektoren schreiben
*-
h_write   move.l  8(sp),6(sp)         ; buf
          move    14(sp),10(sp)       ; count.l = count.w
          move.l  16(sp),12(sp)       ; recno.l = recno.l
          jmp     hd_write


