*+
*     dmaboot.s
*
*        Neuer DMA-Bootloader im ROM fr OMTI-Kontroller
*        Die Routine dmaboot ($fc04a8 dt. ROMTOS) wird gegen
*        folgende ausgetauscht.
*        Es wird ein besonders langes Timeout von 30 Sekunden
*        benutzt, damit langsam anlaufende Platten auch bedient
*        werden. Der Bootsektor mu in den zweiten Teil des
*        1KByte groen Diskbuffer geladen werden, da der erste
*        Teil durch 'hdbios' zerstrt wird. 
*
*     26.01.88  Achim Freimann
*-

flock     equ     $43e            ; Floppy-VBL sperren
_dskbufp  equ     $4c6            ; Zeiger auf 1k Diskbuffer
_hz_200   equ     $4ba            ; 200Hz Zhler
magic     equ     $1234           ; 'magische' Prfsumme

_read     equ     8               ; Sektor(en) lesen
steprate  equ     1               ; 10 mikrosec.

diskctl   equ     $ffff8604       ; Kontrollerzugriff
fifo      equ     $ffff8606       ; DMA-Steuerregister

          text

*+
*       Bootsektor von Platte lesen und ggf. ausfhren
*-
dmaboot   sub.l   a6,a6           ; fr krzere Adressierung
          lea     diskctl,a5
          lea     2(a5),a4        ; A4 = fifo
          moveq   #1,d4
          moveq   #0,d5
          move    #$c8,d6
          move    #$ca,d7
          st      flock(a6)       ; Floppy-VBL sperren
          bsr.s   hard_res        ; Kontroller rcksetzen
          move.l  _dskbufp(a6),a2
          add.l   #$200,a2
          bsr.s   dmaread         ; Bootsektor lesen
          sf      flock(a6)       ; Floppy-VBL freigeben
          tst     d0
          bne.s   bootfail        ; hatte wohl keine Lust

          lea     (a2),a1         ; Start des Bootsektors
          move    #255,d1         ; fr 256 Worte
          moveq   #0,d0
checksum  add     (a2)+,d0        ; Prfsumme bilden
          dbra    d1,checksum
          cmp     #magic,d0
          bne.s   bootfail        ; Bootsektor nicht ausfhrbar
          jsr     (a1)            ; Bootprogramm anspringen
bootfail  rts

*+
*       Bootsektor lesen (Zylinder 0, Kopf 0, Sektor 0)
*
*       Ziel: zweiter Diskbuffer (_dskbufp zeigt auf einen
*             1 KByte groen Buffer)
*
*       dmaread()
*          D0  = 0 ,falls o.k.
*          D0 != 0 ,falls k.o. (KlockOut?)
*-
dmaread   bsr.s   select
          lea     cmd_tab(pc),a0  ; Beginn Kommandotabelle

          moveq   #5,d2
          moveq   #0,d3
send      bsr.s   wait_req
          bne.s   rd_err          ; timeout
          move.b  (a0)+,d3
          move    d3,(a5)
          dbra    d2,send

          move.l  #30*200,d1      ; 30 Sek. timeout
          bsr.s   wait_l

          lea     (a2),a3         ; zweiter Diskbuffer
          move    #511,d2         ; Bytezhler
rd_byte   bsr.s   wait_req
          bne.s   rd_err          ; -> timeout
          move    (a5),d0
          move.b  d0,(a3)+
          dbf     d2,rd_byte
          bsr.s   status          ; Statusbyte lesen

rd_err    move    #$80,(a4)       ; DMA wieder auf Floppy
          tst     d0
          rts

*+
*       Auf bertragungsanforderung warten
*
*         wait_req: max. 1/10 Sekunde warten
*         wait_l:   max. D1.l * 1/200 Sekunden warten
*
*         -> timeout : D0.l = -1
*            ok      : D0.l = 0
*-
wait_req  moveq   #$7f,d1         ; 1/10 sec
wait_l    bsr.s   st_port         ; Statusport anwhlen
          add.l   _hz_200(a6),d1
_wait     move    (a5),d0
          and     d4,d0           ; REQ-Bit testen
          bne.s   wait_ok         ; gesetzt = NE
          cmp.l   _hz_200(a6),d1  ; Zeit abgelaufen ?
          bne.s   _wait
          bsr.s   d_port          ; Datenport anwhlen
          moveq   #-1,d0
          rts

wait_ok   bsr.s   d_port          ; Datenport anwhlen
          moveq   #0,d0
          rts

*+
*       Statusbyte lesen
*-
status    bsr.s   wait_req
          move    (a5),d0         ; Fehlerbyte lesen
          and     #$ff,d0
          tst     d0
          rts

*+
*       Kontroller selektieren
*-
select    bsr.s   sel_port
          move    d5,(a5)         ; Select

*+
*       Datenport whlen
*-
d_port    move    d7,(a4)
          move    d5,(a5)
          move    d6,(a4)
          rts

*+
*       Status-/Resetport whlen
*-
st_port   move    d7,(a4)
          move    d4,(a5)
          move    d6,(a4)
          rts

*+
*       Konfigurations-/Selectport whlen
*-
sel_port  move    d7,(a4)
          move    #2,(a5)
          move    d6,(a4)
          rts

*+
*       Kontroller rcksetzen
*-
hard_res  move    #100,d0         ; ~ 125 usek. Verzgerung
          move    d7,(a4)
          move    #$80,(a5)       ; Reset-Leitung setzen ...
res_loop  dbra    d0,res_loop
          move    d5,(a5)         ; ... und zurcksetzen
          move    d6,(a4)
res_lp    dbra    d0,res_lp       ; Verzgerung, damit der OMTI
          rts                     ; den Reset verarbeiten kann.


*+
*       Kommando fr Kontroller
*-
cmd_tab   dc.b    _read           ; Kommando
          dc.b    0               ; Kopf
          dc.b    0               ; Sektornummer
          dc.b    0               ; Zylinder
          dc.b    1               ; Anzahl der Sektoren
          dc.b    steprate        ; Kontrollbyte


