/*                                                          Achim Freimann
      hdpart.c
                  Festplattenpartitionierer

      Das Programm mu mit 'cbind' und 'hdbios' gelinkt werden !

      Lattice C V3.03.04

      16.01.88  af  v2.1
      15.03.88  af  v2.2
*/

#include <stdio.h>
#include <portab.h>
#include <osbind.h>

#define esc 0x1b
#define bs 8
#define lf 10

extern super_on(),superoff();
extern h_read(),h_write();
extern install();

/*    Struktur einer Partitionsinformation     */

struct PARTINFO
{  BYTE p_flg;      /*  Part.-Flag  <>0 : Part. existiert., $81= Bootpart.  */
   BYTE p_id[3];    /*  Partitionsname. Fr TOS = 'GEM'  */
   LONG p_st;       /*  Start der Partition (logischer Sektor)  */
   LONG p_cnt;      /*  Partitionslnge in Sektoren  */
};

/*   Allgemeine Struktur des Bootsektors
     Mit den Worteintrgen hat man seine liebe Not. Sie sind halt so
     intel(ligent) abgelegt  */

struct BOOTSECT
{  WORD branch;          /*  Verzweigung zum Bootprogramm  */
   BYTE filler[6];       /*  Identifikationsname  */
   BYTE serial[3];       /*  Seriennummer vom Formatierer geschrieben  */
   BYTE bpsl;            /*  Bytes/Sektor  low,high  */
   BYTE bpsh;
   BYTE spc;             /*  Sektoren/Cluster  */
   BYTE resl;            /*  Anzahl reservierter Sektoren   low,high  */
   BYTE resh;
   BYTE nfats;           /*  Anzahl der FATs  */
   BYTE ndirsl;          /*  Anzahl der Directory-Eintrge   low,high  */
   BYTE ndirsh;
   BYTE nsectsl;         /*  Anzahl der Sektoren   low,high  */
   BYTE nsectsh;
   BYTE media;           /*  Media-Byte  (IBM/MS hatten es ja so gerne !!!) */
   BYTE spfl;            /*  Sektoren/FAT   low,high  */
   BYTE spfh;
   BYTE sptl;            /*  Sektoren/Spur   low,high  */
   BYTE spth;
   BYTE nsidesl;         /*  Anzahl der Seiten  low,high!  */
   BYTE nsidesh;         /*   Die hatten aber groes vor !  */
   BYTE nhidl;           /*  Anzahl der versteckten Sektoren   low,high  */
   BYTE nhidh;
   WORD bootcode[210];   /*  Hier kann das Bootprogramm stehen  */
   LONG hd_siz;          /*  Gesamtgre der Festplatte in Sektoren  */
   struct PARTINFO p[4]; /*  Platz fr vier Partitionseintrge  */
   LONG bsl_st;          /*  Startsektor der Liste ber defekte Sektoren  */
   LONG bsl_cnt;         /*  Anzahl der defekten Sektoren  */
   WORD magic;
}b;


main()
{
   int choice,sectors;

   super_on();
   install();                /*   Treiber initialisieren   */
   superoff();
   rd_sect(0,&b);
   sectors=b.sptl*b.nsidesl; /*   Anzahl der Sektoren pro Zylinder */

   for(;;)
   {
      info(sectors);
      choice=menu();
      switch(choice)
      {  case '1':
            create(sectors); /*  Part. anlegen  */
            break;
         case '2':
            erase();         /*  Part. lschen  */
            break;
         case '3':
            form();          /*  Part. logisch formatieren  */
            break;
         case '4':
            boot();          /*  Bootflag setzen  */
            break;
         case '5':
            reboot();        /*  Warmstart durchfhren  */
      }
   }
}

menu()
{
   int c;

   printf("%c%c\n\n",esc,'H');
   puts("    Partitionierer fr Festplatte an PC/XT-Kontroller   v2.2");
   puts("    --------------------------------------------------------\n\n");
   puts("       1. Partition anlegen / ndern");
   puts("       2. Partition lschen");
   puts("       3. Partition logisch formatieren");
   puts("       4. Bootpartition whlen");
   puts("       5. Ende\n");
   printf("      Bitte gewnschte Nummer eingeben !      -  %c",bs);

   while((c=getchar()) < '1' || c > '5' );
   return(c);
}

create(sectors)    /*  Partition anlegen  */
int sectors;
{
   int c,count,sect,stcyl,cylcnt;

   rd_sect(0,&b);
   while((c=getchar()) != lf);     /* Tastaturbuffer leeren */
   printf("\n  Nummer der Partition [ 0 - 3 ] ?  - ");
   if((c=getchar()) >= '0'-1 & c < '4' )
   {
      c -= '0';
      b.p[c].p_flg=0;        /*  Part. vorrbergehend lschen */
      get_cyl(&stcyl,&cylcnt,sectors);
      b.p[c].p_st=sectors*stcyl;
      b.p[c].p_cnt=sectors*cylcnt;
      printf("  ID [ fr TOS: GEM ] ?  ");
      scanf("%3s",&b.p[c].p_id);
      b.p[c].p_flg=1;        /*  Flag fr existierende Part.  */

      wr_sect(0,&b);         /*  Info in Sektor 0 eintragen  */

      sect=b.p[c].p_st;
      count=b.p[c].p_cnt;
      rd_sect(sect,&b);
      creat_boot();          /*  Bootsektor fr die neue Part. erzeugen  */
      b.nsectsl=count & 0xff;
      b.nsectsh=(count >> 8 ) & 0xff;
      b.spfl=count/512 + (count%512 ? 1 : 0);  /*  FAT-Gre ausrechnen  */
      b.spfh=0;
      wr_sect(sect,&b);
   }
}

get_cyl(stcyl,cylcnt,sectors)
int *stcyl,*cylcnt,sectors;
{
   char *msg;
   int i,input,err,maxcyl;
   msg="  ? Zylinder liegt innerhalb der Partition";
   maxcyl=b.hd_siz/sectors-1-2;
                         /*  ^- falls am Ende der Platte
                                Ausweichspuren liegen    */
   do
   {
      err=0;
      printf("  Startzylinder [ 0 =< x ] ?  ");
      scanf("%d",&input);

      if(input<0 || input >= maxcyl)
      {  printf("  ? Zylindernummer auerhalb des Wertebereichs !\n");
         err=1;
      }
      for(i=0;i<=3;i++)
      {
         if(b.p[i].p_flg != 0)
         {
            if(input >=b.p[i].p_st/sectors &&
               input<=(b.p[i].p_st+b.p[i].p_cnt)/sectors-1)
            {  printf("%s %d !\n",msg,i);
               err=1;
            }
         }
      }
   }
   while (err != 0);
   *stcyl=input;

   do
   {
      err=0;
      printf("  Endzylinder [ 0 < x <= max. Zyl. ] ?  ");
      scanf("%d",&input);

      if(input<=*stcyl || input > maxcyl)
      {  printf("  ? Zylindernummer auerhalb des Wertebereichs !\n");
         err=1;
      }
      for(i=0;i<=3;i++)
      {
         if(b.p[i].p_flg != 0)
         {
            if(*stcyl <= b.p[i].p_st/sectors)
               if(input >= b.p[i].p_st/sectors)
               {
                  printf("%s %d !\n",msg,i);
                  err=1;
               }
         }
      }
   }
   while (err != 0);
   *cylcnt=input+1-*stcyl;
   if(*cylcnt*sectors > 0x8000)
      printf("  ? Warnung: TOS verwaltet max. 16 MByte !\n");
}

erase()     /*  Partition lschen  */
{
   int c;

   rd_sect(0,&b);
   while((c=getchar()) != lf);     /* Tastaturbuffer leeren */
   printf("\n  Nummer der Partition [ 0 - 3 ] ?  - ");
   if((c=getchar()) >= '0'-1 & c < '4' )
   {  b.p[c-'0'].p_flg=0;     /*  Part. Flag lschen  */
      wr_sect(0,&b);      
   }
}
form()        /*  Partition logisch formatieren  */
{
   int c,d,i,sect;
   BYTE buf[512];

   for(i=0;i<512;i++)
      buf[i]=0;

   rd_sect(0,&b);
   while((c=getchar()) != lf);     /* Tastaturbuffer leeren */
   printf("\n  Nummer der Partition [ 0 - 3 ] ?   - ");
   if((c=getchar()) >= '0'-1 & c < '4' )
      if(b.p[c-'0'].p_flg)
      {  while((d=getchar()) != lf);     /* Tastaturbuffer leeren */
         puts("  Achtung: FAT und Directory werden gelscht !");
         printf("  Einverstndnis mit 'j'  ");
         if((d=getchar()) == 'j')
         {  sect=b.p[c-'0'].p_st;
            rd_sect(sect,&b);

            /*  FAT und Directory lschen  */
            for(i=b.spfl*2+(b.ndirsl>>4)+1;i>=0;i--)
            {  sect += 1;        /*  ^ da sollte /16 stehen, aber der       */ 
               wr_sect(sect,&buf);   /*  Compiler hat halt einen Fehler ... */
            }
         }
      }
}

boot()      /*  Bootpartition auswhlen  */
{
   int c,i;
 
   rd_sect(0,&b);
   while((c=getchar()) != lf);     /* Tastaturbuffer leeren */
   printf("\n  Nummer der neuen Bootpartition [ 0 - 3 ] ?  - ");
   if((c=getchar()) >= '0'-1 & c < '4' )
      if(b.p[c-'0'].p_flg)
      {  for(i=0;i<4;i++)     /*  alle alten Bootflags lschen  */
            b.p[i].p_flg=b.p[i].p_flg & 0x7f;
         b.p[c-'0'].p_flg=b.p[c-'0'].p_flg | 0x80;  /* neues Bootflag setzen */
      }
   wr_sect(0,&b);
}

reboot()    /*  Warmstart durchfhren  */
{
   int c;

   while((c=getchar()) != lf);     /* Tastaturbuffer leeren */
   puts("\n    Es wird jetzt ein Warmstart durchgefhrt");
   printf("    Einverstndnis mit 'j'  ");
   if((c=getchar()) != 'j')
      return(0);
   Supexec(0xfc0000);      /* Boot-ROM anspringen (sehr elegant, spart den
                                         Druck auf das kleine Knpfchen !) */
}

info(sectors)      /*  Partitionsinformationen anzeigen  */
int sectors;
{
   int i,j;
   
   rd_sect(0,&b);
   printf("%c%c",esc,'E');
   printf("%c%c%c%c",esc,'Y',32+16,32+50);   /* Cursor positionieren */
   printf("Nr.  ID   Boot  Start  Ende");

   for(i=0;i<4;i++)
   {
      printf("\n%c%c%c%c",esc,'Y',32+18+i,32+51);   /* Cursor pos. */
      printf("%c   ",b.p[i].p_flg?'0'+i:'-');
      if(b.p[i].p_flg)
      {  for(j=0;j<3;j++)
            putchar(b.p[i].p_id[j]);
         printf("    %c",b.p[i].p_flg&0x80?'+':'-');
         printf("    %3d    %3d",b.p[i].p_st/sectors,
                 (b.p[i].p_cnt+b.p[i].p_st)/sectors-1);
      }
    }
    printf("\n%c%c%c%c",esc,'Y',32+23,32+50);   /* Cursor pos. */
    printf("hchster Zylinder: %3d",b.hd_siz/sectors-3);
}

creat_boot()    /*  Bootsektor erzeugen  */
{
   b.branch=0x601c;           /* bra.s bootcode */
   b.bpsl=0;                  /* Sektorlnge 512 Bytes */
   b.bpsh=2;
   b.spc=2;                   /* Die sen kleinen 1KByte Cluster ... */
   b.resl=1;                  /* ein reservierter Sektor (Bootsektor) */
   b.resh=0;
   b.nfats=2;
   b.ndirsl=112;              /* 112 Eintrge im Wurzelverz. = 7 Sektoren */
   b.ndirsh=0;
   b.media=0;                 /* Was wohl BigBlue hier rein schreibt ? */
   b.nhidl=0;                 /* keine versteckten Sektoren */
   b.nhidh=0;
   b.bootcode[0]=0x4e75;      /* rts */
}

rd_sect(sect,buf)    /*  einen Sektor lesen  */
int sect;
int buf;
{
   super_on();
   h_read(0,buf,1,sect,0);    /* Der Aufruf ist hnlich rwabs(), flag und dev */
   superoff();                /* werden aber nicht mehr bercksichtigt */
}

wr_sect(sect,buf)    /*  einen Sektor schreiben  */
int sect;
int buf;
{
   super_on();
   h_write(0,buf,1,sect,0);   /* s.o. */
   superoff();
}

