*+
*     pbios.s
*
*         Diese Datei enthlt  _n_u_r_ die minimalen
*         Ausgaberoutinen fr Festplatte mit OMTI 5520A am DMA-Port
*         Der komplette Treiber befindet sich in 'hdbios.s'
*-

*       Einige Definitionen

drv_num   equ     '   C'-'   A'       ; erste Laufwerksnummer
gemdos    equ     1
super     equ     $20                 ; Supervisormodus ein-/ausschalten
flock     equ     $43e                ; Variable zum sperren von FDC-Aktionen
_dskbufp  equ     $4c6                ; Zeiger auf 1k Diskbuffer
_hz_200   equ     $4ba                ; 200Hz Zhler

fat_ind   equ     1                   ; Flag fr 16 Bit FAT Eintrge
steprate  equ     1                   ; 10 mikrosec.
* 0=  3ms, 1= 10us, 2= 25us, 3= 50us, 4=200us, 5= 70us, 6=  3ms, 7=  3ms

*       Adressen vom DMA-Chip u..

diskctl   equ     $ffff8604           ; Kontrollerzugriff/Sektorzhler
fifo      equ     $ffff8606           ; DMA-Steuerregister
dmahigh   equ     $ffff8609           ; DMA-Basis Register
dmamid    equ     $ffff860b
dmalow    equ     $ffff860d

gpip      equ     $fffffa01           ; MFP-Parallelport (IRQ-Port)

*       Befehle des OMTI 5520 Kontrollers

_recal    equ     1                   ; Spur Null anfahren
_read     equ     8                   ; Sektor(en) lesen
_init     equ     $c                  ; Laufwerksparameter setzen


*       GEMDOS Fehlercodes

OK        equ     0                   ; Kein Fehler
ERROR     equ     -1                  ; Allgemeiner Fehler
RD_ERR    equ     -11                 ; Lesefehler


*       etwas fr den Linker ...

          xdef    hd_read
          xdef    hd_write
          xdef    hd_init
          xdef    hd_recal
          xdef    send_cmd
          xdef    wait_req
          xdef    status
          xdef    wait_l
          xdef    super_on
          xdef    superoff
          xdef    reset
          xdef    install
          xdef    hard_res


          text

*+
*       Supervisormodus einschalten
*-
super_on  clr.l   -(sp)
          move    #super,-(sp)
          trap    #gemdos
          addq.l  #6,sp
          lea     save_ssp(pc),a0
          move.l  d0,(a0)             ; SSP merken
          rts

*+
*       Supervisormodus ausschalten
*-
superoff  move.l  save_ssp(pc),-(sp)  ; alten SSP zurck
          move    #super,-(sp)
          trap    #gemdos
          addq.l  #6,sp
          rts

*+
*       Lesen von Sektoren
*-
buf       set     10
count     set     14
recno     set     16
*
hd_read   link    a6,#0
          movem.l d1-d5/a3-a5,-(sp)
          lea     diskctl,a5
          lea     2(a5),a4
          st      flock               ; Floppy VBL sperren
          move.l  recno(a6),d5
          move.l  buf(a6),a3
          moveq   #0,d4
          move    count(a6),d4
          move    #32,d3              ; max. Anzahl der Sektoren
          divu    #32,d4
          bra     rd_loop

rd_block  move    #steprate,-(sp)     ; Kontroll-Byte
          clr     -(sp)
          move.b  d3,1(sp)            ; Sektorzhler
          move.l  d5,d0
          bsr     rec_calc            ; log. Sektor -> Zylinder, Kopf, Sektor
          movem   d0-d2,-(sp)
          move    #_read,-(sp)
          bsr     send_dma            ; DMA-Kommando senden
          lea     12(sp),sp
          tst     d0
          bne     rd_err1

          bsr     mask
          move    #3,(a5)             ; IRQ- & DMA-Maske setzen
          move    #$ca,(a4)
          move    #4,(a5)             ; DMA-Richtung: lesen

          move.l  a3,-(sp)
          move.b  3(sp),dmalow        ; bertragungsadresse setzen
          move.b  2(sp),dmamid
          move.b  1(sp),dmahigh
          addq.l  #4,sp

          move    #$98,(a4)           ; DMA-Status durch 'Toggeln' lschen
          move    #$198,(a4)
          move    #$98,(a4)

          moveq   #0,d0
          move.b  d3,d0
          move    d0,(a5)             ; Blockzhler fr DMA-Chip

          move    #$8,(a4)
          bsr     wait_dma            ; auf IRQ warten
          bne     rd_err1
          bsr     mask
          clr     (a5)                ; IRQ- & DMA-Maske lschen
          bsr     status              ; Statusbyte lesen
          bne     rd_err1

          clr.l  -(sp)
          move.b  dmalow,3(sp)        ; DMA-Endadresse holen
          move.b  dmamid,2(sp)
          move.b  dmahigh,1(sp)
          move.l  (sp)+,a3

          add.l   #32,d5              ; Startsektor des nchsten Blocks
rd_loop   subq    #1,d4
          bge     rd_block            ; noch nicht alle 32-Sektor-Blcke ?
          cmp     #-1,d4              ; alle Blcke gelesen ?
          blo     rd_err

          swap    d4
          tst     d4
          beq     rd_err
          move    d4,d3               ; restliche Sektoren lesen
          swap    d4
          bra     rd_block

rd_err1   move.l  #RD_ERR,d0          ; Lesefehler

rd_err    move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d5/a4-a5
          unlk    a6
          rts

*+
*       logischen Sektor in Zylinder, Kopf und Sektor umrechnen
*-
rec_calc  divu    sectors(pc),d0
          swap    d0
          move    d0,d1               ; Sektor
          clr     d0
          swap    d0
          divu    heads(pc),d0        ; d0.w = C0-C9
          move.b  d0,d2               ; Zylinder C0-C7
          and     #$ff,d2
          clr.b   d0
          lsr     #2,d0               ; Zylinder C8-C9
          or.b    d0,d1               ; Sektor .oder C8-C9
          swap    d0                  ; Kopf
          rts

*+
*       Laufwerk auf Spur Null fahren und max. 10 Sekunden warten
*         Nach einem Reset wird der erste Seek-Vorgang mit niedrigster
*         Steprate durchgefhrt wobei es bis zu 10 Sek. dauern kann, falls
*         der Kopf auf einer hohen Zylindernummer (evtl. Parkzone) steht.
*
*         hd_recal()
*-
hd_recal  link    a6,#0
          movem.l d1-d3/a4-a5,-(sp)
          lea     diskctl,a5
          lea     2(a5),a4
          st      flock               ; Floppy VBL sperren
          bsr     reset               ; Kontroller rcksetzen

          move    #steprate,-(sp)
          clr.l   -(sp)
          clr.l   -(sp)
          move    #_recal,-(sp)
          bsr     send_cmd
          lea     12(sp),sp
          move.l  #2000,d1            ; 10 Sek. timeout
          bsr     wait_l
          bsr     status              ; Statusbyte lesen

recal_er  move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d3/a4-a5
          unlk    a6
          rts

*+
*       Laufwerksparameter setzen
*-
param     set     8
*
hd_init   link    a6,#0
          movem.l d1-d3/a4-a5,-(sp)
          lea     diskctl,a5
          lea     2(a5),a4
          st      flock               ; Floppy VBL sperren
          bsr     reset               ; Kontroller rcksetzen

          clr.l   -(sp)
          clr.l   -(sp)
          clr     -(sp)
          move    #_init,-(sp)
          bsr     send_cmd
          bne     init_err            ; -> timeout
          lea     12(sp),sp

          moveq   #0,d0
          move.l  param(a6),a0        ; Zeiger auf Parametertabelle
          moveq   #7,d3               ; Bytezhler
ini_byte  bsr     wait_req
          bne     init_err            ; -> timeout
          move.b  (a0)+,d0
          move    d0,(a5)
          dbra    d3,ini_byte
          bsr     status              ; Statusbyte lesen

init_err  move    #$80,(a4)           ; DMA wieder auf Floppy
          sf      flock               ; Floppy VBL freigeben
          movem.l (sp)+,d1-d3/a4-a5
          unlk    a6
          rts

*+
*       Kommando an Kontroller bertragen
*-
send_cmd  link    a6,#0
          movem.l d1-d2/a3-a5,-(sp)
          lea     diskctl,a5
          lea     2(a5),a4
          bsr     select              ; Kontroller selektieren

          moveq   #5,d2               ; Anzahl Datenbytes (=6)
          lea     8(a6),a3            ; Zeiger auf Parameter
scmd      bsr     wait_req
          bne     scmd_end            ; Zeitbegrenzung berschritten
          move    (a3)+,(a5)          ; Kommando bzw. Parameter
          dbra    d2,scmd
          move.l  #400,d1             ; 2.0 Sek warten
          bsr     wait_l
          bne     scmd_end
          moveq.l #0,d0

scmd_end  tst     d0
          movem.l (sp)+,d1-d2/a3-a5
          unlk    a6
          rts

*+
*       DMA-Kommando an Kontroller bertragen
*-
send_dma  link    a6,#0
          movem.l d1-d2/a3-a5,-(sp)
          lea     diskctl,a5
          lea     2(a5),a4
          bsr     select              ; Kontroller selektieren

          moveq   #5,d2               ; Anzahl Datenbytes (=6)
          lea     8(a6),a3            ; Zeiger auf Parameter
sdma      bsr     wait_req
          bne     sdma_end            ; Zeitbegrenzung berschritten
          move    (a3)+,(a5)          ; Kommando bzw. Parameter
          dbra    d2,sdma
          moveq.l #OK,d0

sdma_end  tst     d0
          movem.l (sp)+,d1-d2/a3-a5
          unlk    a6
          rts

*+
*       Auf bertragungsanforderung warten
*-
wait_req  move.l  #20,d1              ; 1/10 sec
wait_l    bsr     st_port             ; Statusport anwhlen
          add.l   _hz_200,d1
_wait     move    (a5),d0
          and     #1,d0               ; REQ-Bit testen
          bne     wait_ok             ; gesetzt = NE
          cmp.l   _hz_200,d1          ; Zeit abgelaufen ?
          bne     _wait
          bsr     d_port              ; Datenport anwhlen
          move.l  #ERROR,d0
          rts

wait_ok   bsr     d_port              ; Datenport anwhlen
          moveq.l #OK,d0
          rts

*+
*       Auf DMA-bertragung warten
*-
wait_dma  moveq.l #OK,d0
          move.l  #200,d1             ; 1 sec
wait_ld   add.l   _hz_200,d1
_waitd    btst    #5,gpip             ; IRQ testen
          beq     dma_ok              ; Hurra, er ist da !
          cmp.l   _hz_200,d1          ; Zeit abgelaufen ?
          bne     _waitd
          bsr     reset               ; Kontroller rcksetzen
          move.l  #ERROR,d0
dma_ok    rts

*+
*       Hard-Reset durch Setzen der Reset-Leitung
*-
hard_res  st      flock
          move    #100,d0             ; => ~ 125 mikrosek. Verzgerung
          move    #$ca,fifo
          move    #$80,diskctl        ; Reset-Leitung setzen ...
res_loop  dbra    d0,res_loop
          clr     diskctl             ; ... und zurcksetzen
          move    #$c8,fifo
res_lp    dbra    d0,res_lp           ; Verzgerung, damit der OMTI den Reset
          sf      flock               ;             in Ruhe verarbeiten kann.
          rts

*+
*       Statusbyte lesen
*-
status    bsr     wait_req
          move    (a5),d0             ; Fehlerbyte lesen
          and     #$ff,d0
          tst     d0
          rts

*+
*       Kontroller rcksetzen (Soft-Reset)
*-
reset:    bsr     st_port
          clr     (a5)   
          bra     d_port

*+
*       Kontroller selektieren
*-
select    bsr     sel_port
          clr     (a5)               ; Selectport beschreiben (= Select)
          bra     d_port

*+
*       Datenport whlen
*-
d_port    move    #$ca,(a4)
          clr     (a5)   
          move    #$c8,(a4)
          rts

*+
*       Status-/Resetport whlen
*-
st_port   move    #$ca,(a4)
          move    #1,(a5)   
          move    #$c8,(a4)
          rts

*+
*       Konfigurations-/Selectport whlen
*-
sel_port  move    #$ca,(a4)
          move    #2,(a5)   
          move    #$c8,(a4)
          rts

*+
*       DMA/INT-Maskenport whlen
*-
mask      move    #$ca,(a4)
          move    #3,(a5)   
          move    #$c8,(a4)
          rts

heads     dc.w    1                   ; Anzahl der Kpfe (wird initialisiert)
sectors   dc.w    1                   ; Anzahl der Sektoren/Spur     "
save_ssp  ds.l    1                   ; Zwischenspeicher fr SSP

          end

